/****************************************************************
*FILE :				usercode.c	                                *
*                                                               *
*					application specific routines				*
*                                                               *
* Copyright [C] 2010 by											*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/

/************************* INCLUDES ****************************/
#include "stdafx.h"
#include "include_all.h"

/*************************** DEFINES ****************************/

/******************* EXTERNAL VARIABLES ************************/

/******************* FUNCTION DECLARATIONS *********************/

extern void RootSetLanguage(void) ;		// in MRoot of std_system.rad

/********************* GLOBAL VARIABLES ************************/

/*****************************************************************************/


/****************************************************************
*FUNCTION NAME:		doUserFunc									*
*													            *
*DESCRIPTION:      user Functions of radCASE application		*
*                                                               *
*PARAMETER:        function number                              *
*                                                               *
*RETURN:           always 0                                     *
*                                                               *
****************************************************************/
short doUserFunc(short nr)
{  
	nr = nr ;		// derzeit keine Function implementiert			
	return(0);
} 


/****************************************************************
*FUNCTION NAME:		UserEnd										*
*													            *
*DESCRIPTION:      Procedures called at the end of the program	*
*                                                               *
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
void UserEnd(void)
{
}

/****************************************************************
*FUNCTION NAME:		do_perm										*
*													            *
*DESCRIPTION:		Task for all permanent process functions	*
*					required only for simulation, part of		*
*					API for target 								*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
#ifndef ___CTR___ 
void do_perm(void)
{
	SysPermInit();
	UserPermInit();
	
	for(;;) //while (TRUE)
	{
		SysPerm();
		UserPerm();
		executePERM();
		Sleep(10);       
	}
}
#endif

/****************************************************************
*FUNCTION NAME:		do_eeprom									*
*													            *
*DESCRIPTION:		Task EEPROM Handling						*
*					required only for simulation, part of		*
*					API for target 								*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
#ifndef ___CTR___ 
void do_eeprom(void)
{
	for(;;)
	{
		PersistHandling();
		Sleep(10);
	}
}
#endif


/****************************************************************
*FUNCTION NAME:		ModulInit									*
*													            *
*DESCRIPTION:		Initialisation of global module pointers	*
*					required by radCASE							*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
void ModulInit(void)
{
// this function call is required by every Design using MRoot of std_system.rad 
#if ((NO_EMB_HMI == 0) || (RD_CE_ACCESS != 0))
	RootSetLanguage() ;
#endif
}

/****************************************************************
*FUNCTION NAME:		UserInit									*
*													            *
*DESCRIPTION:		Application specific init functions			*
*					required by radCASE							*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
void UserInit(void)
{
}

/****************************************************************
*FUNCTION NAME:		UserPermInit								*
*													            *
*DESCRIPTION:		Application specific perm init functions	*
*					required by radCASE							*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
void UserPermInit(void)
{ 
	executePARAMUPD();
}

/****************************************************************
*FUNCTION NAME:		UserPerm									*
*													            *
*DESCRIPTION:		Application specific perm code				*
*					required by radCASE							*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
void UserPerm(void)
{
}


/****************************************************************
*FUNCTION NAME:		UserInter									*
*													            *
*DESCRIPTION:		Application specific Timer interrupt code	*
*					required by radCASE							*
*																*
*PARAMETER:        none				                            *
*                                                               *
*RETURN:           none		                                    *
*                                                               *
****************************************************************/
void UserInter(void)
{
}


/****************************************************************
*FUNCTION NAME:		key_pressed / key_pressed_stat				*
*													            *
*DESCRIPTION:		key manipulation functions					*
*																*
*PARAMETER:			key				                            *
*                                                               *
*RETURN:			manipulated key                             *
*                                                               *
****************************************************************/
short key_pressed(short key)
{
#if (T_KEY_ABORT)
  if (testAbort() == 1)
    key = abortKey;
  else
	ResetAbortTimer();
#endif  
 	return(key);
}

short key_pressed_stat(M_D_SCR short key)
{
#if (T_KEY_ABORT)
  if (testAbort() == 1) 
	  key = abortKey;
#endif
	return(key);
}

#ifndef ___CTR___
/****************************************************************
*FUNCTION NAME:		mapSimKey									*
*													            *
*DESCRIPTION:		key manipulation functions					*
*																*
*PARAMETER:			key				                            *
*                                                               *
*RETURN:			manipulated key                             *
*                                                               *
****************************************************************/
short mapSimKey(M_D_SCR short c)
{                      
	if ((screen->ActKeyMap == KEYMAP_SYSTEM) || (screen->ActKeyMap == KEYMAP_EDIT))
	{
		switch(c)
		{
			case F1: c = ESC;	break;	
			case F2: c = AKEY1;	break;				
			case F3: c = CR;	break;	
		}
	}
	else
	{
		switch(c)
		{
			case F1: c = ESC;	break;	
			case F2: c = F2;	break;				
			case F3: c = CR;	break;	
		}
	}
	return(c);
}
#endif // #ifndef ___CTR___

/****************************************************************
*													            *
*         Task Table for radCASE perm function					*
*													            *
****************************************************************/
#ifndef ___CTR___
CTR_TASK ApplTaskTable[] = {
	{NULL, &do_perm,				TASK_TYPE_LOOP,  100, 3},
	{NULL, &do_eeprom,				TASK_TYPE_LOOP,  100, 3},
	{NULL, NULL,0,0,0}					// Endekennung
};
#endif
