#ifndef __action_types_h__
#define __action_types_h__

#if USE_ULONG4OFFSETS != 0
	#define XPTR    unsigned long
    #define XSTR    unsigned long
	#define RD_NOBINDATA 0xFFFFFFFF
#else
	#define	XPTR    unsigned short
    #define XSTR    unsigned short
	#define RD_NOBINDATA 0xFFFF
#endif

#define ATYPE_CFUNC				1
#define ATYPE_SET				2
#define ATYPE_OPEN				3
#define ATYPE_CLOSE				4
#define ATYPE_RES				5
#define ATYPE_INC				6
#define ATYPE_DEC				7
#define ATYPE_PUT				8
#define ATYPE_PASSWORD			9
#define ATYPE_PASSWORD_CLEAR	10
#define ATYPE_INV				11
#define ATYPE_EDIT				12
#define ATYPE_ASKOK				13
#define ATYPE_GOTOPAGE			14
#define ATYPE_GOTOSURFACE		15
#define ATYPE_RESET				16
#define ATYPE_USERFUNC			17
#define ATYPE_CALLSURFACE		18
#define ATYPE_RETURN			19
#define ATYPE_ACN_PROC			20
#define ATYPE_COPY				21


typedef struct				// allg. Description eines Objektes
{
	XSTR	name;			// Name der Modulinstanz
	XSTR	instdesc;		// Description (eigentlich Info) der Modulinstanz
} DModul;

// General access will be used by PasswordClear/Close/Return too
typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
} DAction;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	elemindex;		// Index of element in element table
} DElemAction;

// Beginnning has to match DElemAction because DActionPut will also be
// used as DElemAction
typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	long	wert;			// value to set element to
	short	elemindex;		// Index of element in element table
} DActionPut;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	funcindex;		// Index of function in function table
} DActionCFunc;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	funcindex;		// Index of function in function table
	XSTR	desc; 			// Index to text of description
	XSTR	name;			// Index to text of function name
} DActionProc;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	level;			// Needed password level
	short	numJump;		// Number of actions to jump over if password level not sufficient
} DActionPassword;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	numJump;		// Number of actions to jump over if dialog is canceled (question type in topmost 2 Bits)
} DActionAskOk;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	surfacenr;		// Surface number to navigate to
} DActionChangeSurface;

typedef struct
{
	XPTR	next;			// Offset of next action
	short	actiontype;		// Actiontype
	short	surfacenr;		// Surface number to navigate to
	short	modindex;		// Index of module in module table
} DActionOpen;

union UAction
{
	DAction action;
	DElemAction elemAction;
	DActionPut actionput;
	DActionCFunc actioncfunc;
	DActionProc actionproc;
	DActionPassword actionpassword;
	DActionAskOk actionaskok;
	DActionChangeSurface actionchangesur;
	DActionOpen actionopen;
};

#endif // __action_types_h__
