#ifndef _COMINTERFACE_H_
#define _COMINTERFACE_H_

// Burst defines
#define RD_BURST_KEEPALIVE			0
#define RD_BURST_DATAPACKET			1

// Visualization notifications
#define RD_VIS_NOTIFY_PARCHANGED	0x40
#define RD_VIS_NOTIFY_BURST			0x80

//
// CMD1-Commands
//
#define VCM_HANDSHAKE				0x11	// Perform handshake to init communication
#define VCM_GET_STATUS				0x12	// I/O-, Flag- oder Processdata
#define VCM_GET_STATUS_PART			0x13	// partial I/O-, Flag-, Parameter- or Procdata
#define VCM_SEND_KEY				0x14	// simulate pressed key
#define VCM_SEND_TOUCH              0x15    // simulate touch press
#define VCM_GET_PROT_HEADER			0x16	// Deprecated: Initiate Protocol Upload
#define VCM_GET_PROT_BLOCK			0x17	// Deprecated: Upload Protocol Data Record
//#define COM_FLASH_DEL_DONE		0x1C	// old command not supported anymore
#define COM_REC_CALLBACKNUM			0x27	// transmit callback number for Modem
#define COM_SND_CTR_ID				0x28	// send Equipment ID, start timeout for Password
#define COM_CHECK_PSWD				0x29	// Check password
#define COM_HANGUP					0x30	// Hangup modem
#define VCM_QUICKKALIB				0x44	// quick AI calibration
#define VCM_GETPHYSVAL				0x45	// get AI physical value
#define VCM_EXEC_CFUNC				0x46	// execute CFunc
#define VCM_SET_STATUS_PART			0x47	// set I/O-, Flag-, Parameter- or Procdata
//#define RESERVED					0x4C	// Should not be used, because first Byte of LIV-Request
#define VCM_GET_FILESYSINFO			0x50	// Send back file system info structure
#define VCM_FILE_COMMAND			0x51	// Execute file command (specific command in cmd2)
#define VCM_SET_PARAM_BLK			0x88	// set Params (next Block)
#define VCM_SET_FLAG				0x89	// set a single Flag Element
#define VCM_SET_IO					0x8A	// set a single IO Element (DO, AO, DI, AI, CNT)
#define VCM_SND_DISPLAY				0x8B	// send display content
//#define VCM_START_EUROS_GATEWAY	0x8C	// old command not supported anymore
//#define VCM_CMD_REQUEST			0x8D	// old command not supported anymore
//#define VCM_SET_TIME				0x8E	// command not supported anymore
#define VCM_SET_PARAM				0x8F	// set a single Param element
#define VCM_ENABLE_BURST			0x90	// enable burst mode
#define VCM_SET_PROC				0x91	// set a Process Variable
#define VCM_GET_DISPLAY_INFO		0x92	// send information about display
//#define VCM_TRANSPARENT_MODEON	0x93	// old command not supported anymore
#define VCM_GET_ALL_VERSIONS		0x94	// send version information
#define VCM_SET_SYSPARAM			0x95	// set a single SysParam element
#define VCM_SET_SYSTEM_BLK			0x96	// set SysParams (next Block)
#define VCM_SND_MEM2PC				0x97	// send data by direct Address to PC
#define VCM_SND_MEM2CTR				0x98	// send data by direct Address to target
#define VCM_GET_PARAM_BLK			0x99	// Parameter data
//#define VCM_SND_MEM_END			0x9A	// command not supported anymore
#define VCM_GET_INDEXED_DATA		0x9B	// send indexed adressed data
#define VCM_SET_INDEXED_DATA		0x9C	// receive indexed addressed data
#define VCM_DISABLE_BURST			0x9D	// disable burst mode
//#define VCM_SAVE_FILE				0xA0	// old command not supported anymore
//#define VCM_LOAD_FILE				0xA1	// old command not supported anymore
//#define VCM_DELETE_FILE			0xA2	// old command not supported anymore
//#define VCM_GET_DISKINFO			0xA3	// old command not supported anymore
//#define VCM_GET_FILESINFO			0xA4	// old command not supported anymore
#define VCM_SND_BIG_BLK				0xA5	// Deprecated: Send a data block to PC
#define VCM_CMD_APPCMD				0xA6	// application specific command
#define VCM_SET_PROC_BLK			0xA7	// set PROCs (next Block)
//#define VCM_HOST_CMD				0xA8	// command not supported

// Display drawing commands
#define RD_DISPCMD_CLR				1		// command for redraw of screen (c_dis_clr)
#define RD_DISPCMD_BITMAP			2		// command for drawing a bitmap (c_dis_bitmap)
#define RD_DISPCMD_LINE				3		// command for drawing a line (c_dis_line)
#define RD_DISPCMD_RECT				4		// command for drawing a rectangle (c_dis_rect)
#define RD_DISPCMD_CIRC				5		// command for drawing a circle (c_dis_circ)
#define RD_DISPCMD_ARC				6		// command for drawing an arc (c_dis_arc)
#define RD_DISPCMD_CLRRECT			7		// command for clearing a rectangle (c_dis_clrrect)
#define RD_DISPCMD_BITMAPROT		8		// command for drawing a rotated bitmap (c_dis_bitmap_rot)
#define RD_DISPCMD_AREAINIT			9		// command for initializing are for area restoration (c_dis_area_init)
#define RD_DISPCMD_AREASAVE			10		// command for saving an area (c_dis_save_area)
#define RD_DISPCMD_AREARESTORE		11		// command for area restoration (c_dis_restore_area)
#define RD_DISPCMD_CURSORSET		12		// command for setting cursor to a position (c_cur_set)
#define RD_DISPCMD_CURSORMOVE		13		// command for moving cursor to a position (c_cur_move)
#define RD_DISPCMD_CURSORCLEAR		14		// command for removing cursor (c_cur_clear)
#define RD_DISPCMD_STR				15		// command for displaying a string (c_dis_str)
#define RD_DISPCMD_CHAR				16		// command for displaying a character (c_dis_char)
#define RD_DISPCMD_EXTRECT			17		// command for drawing an extended rectangle (c_dis_rectext1)
#define RD_DISPCMD_ERROR			0x7F	// For communicating error status to project monitor (Circular buffer overflow)
#define RD_DISPCMD_FOREGROUND_LAYER 0x80	// Ored to drawing commands, to distinguish layers on receiving side

// Display drawing stati communicated over cmd2
#define RD_DISPCMDSTATUS_REDRAW		0x10		// Redraw was triggered, visu has to throw away all contents still in buffer

// Defines for use in supportedMask of sdInfo structure
#define RD_SDSUP_NONE			0 /* No file system supported */
#define RD_SDSUP_DIRLIST		(1 << 0) /* File system supports directory listings */
#define RD_SDSUP_FILEREAD		(1 << 1) /* File system supports all file read functions */
#define RD_SDSUP_FILEWRITE		(1 << 2) /* File system supports all file write functions */
#define RD_SD_BACKSLASH			(1 << 3) /* File system uses backslash as path seperator (path\to\file) */
#define RD_SDSUP_ALL			(RD_SDSUP_DIRLIST|RD_SDSUP_FILEREAD|RD_SDSUP_FILEWRITE) /* Mask with all supported commands (without SD_BACKSLASH) */

struct sdInfo {
	unsigned short maxPathSize;
	unsigned short maxFilenameSize;
	unsigned char supportedMask;
};

// File transfer request states
enum eFileTransRequest
{
	FT_REQ_NOTHING = 0,
	FT_REQ_READ,
	FT_REQ_WRITE,
	FT_REQ_LIST,				// List all file in current directory
	FT_REQ_FILEEXISTS,			// check if file exists
	FT_REQ_OPEN_FILE = 0x10,	// Open file first (masked with FT_REQ_READ or FT_REQ_WRITE)
	FT_REQ_CLOSE_FILE = 0x20,	// Close file after read/write (masked with FT_REQ_READ or FT_REQ_WRITE)
	FT_REQ_OPEN_LIST = 0x40,	// Start a new directory listing (masked with FT_REQ_LIST)
};

// File transfer replies
enum eFileTransReply
{
	FT_REP_OK = 0,
	FT_REP_RUNNING,				// Current request is still running
	FT_REP_BUSY,				// Request from another communication device is running
	FT_REP_UNSUPPORTED,			// File transfer not supported for communication device
	FT_REP_ERROR = 0x80,		// Error code in lower 7 Bit
};

#endif // _COMINTERFACE_H_