/****************************************************************
*	file RDMacros.h												*
*																*
*	several Macros to call special functions.					*
*	These are the macro definitions for the SIM/VIS.			*
*	target specific version of the macros must be part of		*
*	target directory.											*
*																*
*	Copyright [C] by											*
*		IMACS GmbH												*
*		Mittelfeldstr. 25, D-70806 Kornwestheim					*
*		Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*		E-Mail: info@imacs-gmbh.de								*
****************************************************************/

extern void c_dis_initPixelOutput(short x, short y, short sizex, short sizey);
extern long c_dis_getRGBColor(RD_COLOR	col);
extern void c_dis_setNextPixelPos(short x, short y);
extern void c_dis_incPrintPixels(unsigned char r, unsigned char g, unsigned char b, short count);
extern void c_dis_finishPixelOutput(void);
extern unsigned long c_dis_getPixel(short x, short y);

/************************** DEFINES ****************************/

// macro to define local variables for acessing pixel data
// normally only relevant if application writes directly into video ram
#define DEFPIXELDATA											// unused in SIM/VIS

// macro to initialize area for pixel output, to be called before pixel output is started
// x,y: Position for pixel output, sizex, sizey: Size of pixel output
// Note : x, sizex are positive, y, sizey are negative values
#define INITPIXELOUTPUT(x,y,sizex,sizey)			c_dis_initPixelOutput(x,y,sizex,sizey);

// macro that finishes output of pixel in previously initialized area
#define FINISHPIXELOUTPUT							c_dis_finishPixelOutput() ;

// macro set the pixel address to the correct value
// x,y: starting position for following PUTPIXELINC calls
// Note : x is a positive, y is a negative value
#define SETPIXELADDRESS(x,y)						c_dis_setNextPixelPos(x,y);

// macro to put one pixel to frame buffer and increment address set by SETPIXELADRESS
// r,g,b : colors as unsigned char, count = number of pixels as short
#define PUTPIXELINCRGB(r,g,b,count)					c_dis_incPrintPixels(r,g,b,count);

// macro to put one pixel to frame buffer and increment address set by SETPIXELADRESS
// color : pixel color, count = number of pixels as short
#if BITSPERPIXEL == 16
// color is unsigned short (RGB565)
#define PUTPIXELINC(col,count)					c_dis_incPrintPixels((unsigned char)((col&0xF800) >> 8), (unsigned char)((col&0x07E0) >> 3), (unsigned char)((col&0x001F) << 3),count);
#endif
#if BITSPERPIXEL > 16
// color is unsigned long (RGB888)
#define PUTPIXELINC(col,count)					c_dis_incPrintPixels((unsigned char)((col&0x00FF0000) >> 16), (unsigned char)((col&0x0000FF00) >> 8), (unsigned char)col&0x000000FF,count);
#endif

// macro to get one pixel from the frame buffer (to determine background color)
// returns pixel color information in appropriate format depending on color depth
// Note : x is a positive, y is a negative value
#if BITSPERPIXEL == 16
inline unsigned short convertRGB888toRGB565(unsigned long color)
{
	return (unsigned short)(
		((color & 0xF80000) >> 8)
		| ((color & 0xFC00) >> 5)
		| ((color & 0xF8) >> 3)
	);
}

#define GETPIXEL(x,y)								convertRGB888toRGB565(c_dis_getPixel(x, y))
#endif
#if BITSPERPIXEL > 16
#define GETPIXEL(x,y)								c_dis_getPixel(x, y)
#endif

// macro to get a 32 bit color from an standard color byte
// returns RGB0888 as long
#define GETCOLOR32(col)							c_dis_getRGBColor(col)
