/****************************************************************
*   file FlowCntSetup.h                                         *
*                                                               *
*   Setup data structure for Flow Meter Support in API			*
*	measuring methods :											*
*		- PulseLen : counting the time between 2 pulses			*
*		- PulseCount : Counting pulses in a fixed time gap		*
*                                                               *
* Copyright [C] by												*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/
#ifndef DEF_FLOWCNTSETUP
#define DEF_FLOWCNTSETUP

// Setup data for legacy API implementation of FlowMeters
typedef struct sFlowCntSetup
{
	// Kalibrierwerte
	unsigned long Arate ;			// Anzahl der Zaehlerimpulse fuer ein Incr. des log. Wertes, 0 wenn ungesetzt.
									// Format mit 3 Vor- und 3 Nachkommastellen, also 0.001 .. 999;max 3 gueltige Ziffern
	unsigned long Brate ;			// Anzahl der Incr. des log. Wertes je Zaehlimpuls, 0 wenn ungesetzt
									// Format mit 3 Vor- und 3 Nachkommastellen, also 0.001 .. 999;max 3 gueltige Ziffern
	unsigned char range ;			// Bereichsumschalter fuer das EA-Board, individuell definiert
	// Durchfluss-Parameter
	unsigned long winlen ;			// Zeitfenster des log. Wertes in sek, zulaessige Werte : 1,60,3600 entspr.sek/min/h
	unsigned long zerotime ;		// Zeitfenster fuer Differenz = 0 in ms
	unsigned long FlowDivisor;		// zusaetzlicher Divisor bei der Durchflussermittlung
	// Summen - Parameter
	unsigned long SumDivisor;		// zusaetzlicher Divisor bei der Summenbildung
	unsigned char countingup ;		// 0=counter is counting down (e.g. Fujitsu) / 1 = counter is counting up
	// Diese Parameter sind nur bei lokalen Zaehlern sinnvoll !
	//Tiefpass-Parameter
	unsigned short filter ;			// Zeitkonstante Tiefpass in ticks (10ms), 0= ohne Tiefpass
}FLOWCNTSETUP;


// Setup data for flowmeters
// Setup is called once during init
typedef struct sFlowMeterSetup
{
	// the IO Units assigned to that FlowMeter, at least one of them must be set
	E_AI *	pFlow ;					// Flow, might be NULL (then only absolute Count values are handled)
	E_CNT *	pSum ;					// absolute Volume, might be NULL (then only Flow values are handled)

	// Calibration
	unsigned char TimeBase ;		// Timebase for flow units : 0 = s / 1 = min / 2 = h
	unsigned long SumDivisor;		// additional divisor for sum calculation. must be mutliple of 10 (or 1) Should be used for different vol bases between Flow and Sum
									// e.g. if Flow is in xxx l/h and Volume in xxx m3, then SumDivisor should be 1000
									//      if Flow is in xx.xx l/s and Volume in xxx l, then SumDivisor should be 100
									// Thus more accuracy is achieved compared to setting the Calibration in radCASE, which beside that can generate the same effects
} FLOWMETERSETUP;

// Update data for Flowmeters
// Update is called whenever Parameters are changed
typedef struct sFlowMeterUpdate
{
	// the IO Units assigned to that FlowMeter, at least one of them must be set
	E_AI *	pFlow ;					// Flow, might be NULL (then only absolute Count values are handled)
	E_CNT *	pSum ;					// absolute Volume, might be NULL (then only Flow values are handled)

	// Calibration
	// Arate and Brate are in the logical format xxx.xxx , 1/1000, exactly one of them must be != 0
	unsigned long Arate ;			// No of Pulses per logical increment, if 0, use Brate instead
	unsigned long Brate ;			// No if logical increments per pulse, if 0, use Arate instead

	unsigned char method ;			// Mesuring method :
									// 0	: dynamically switchable (currently not supported) 
									// 1	: PulseLen method
									// 50	: PulseCount method 
									// 51..54 : PulsCount method with extended time window ( *2, *4, *8, *16)  (currently not supported)

	unsigned short debounce ;		// Debounce time in ms (max. 0x3FF), irrelevant in PulseCount method
	unsigned long zerotime ;		// time in ms to set flow to 0. irrelevant in PulseCount method
	unsigned short window ;			// Mesuring window size in ms. Only relevant in PulseCount method
	unsigned short switchvalue ;	// switching value for PulsCount method with extended time window, otherwise don't care
} FLOWMETERUPDATE;



#endif
