#ifndef _FLOWMETER_H_
#define _FLOWMETER_H_

/****************************************************************
*   file FlowMeter.h                                            *
*                                                               *
*   Structure describing FlowMeter								*
*                                                               *
*	Flow is generated as AI, Volume as CNT                      *
*		- PulseLen : counting the time between 2 pulses			*
*		- PulseCount : Counting pulses in a fixed time gap		*
*                                                               *
* Copyright [C] by												*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/
typedef struct sFlowPar
{
	// the IO Units assigned to that FlowMeter
	E_AI *	pFlow ;					// Flow Unit, might be NULL (then only absolute Volume values are handled)
	E_CNT *	pSum ;					// absolute Volume Unit, might be NULL (then only Flow values are handled)

	// Data supplied from Master to Slave on distributed systems
	unsigned short Controlword ;	// bit 10..15 :		
									// 0	  (0x0000)	: PulseCount method 
									// 1..50  (0x0400 .. 0xC800) : PulseLen method, value 1..50 is factor (ms) for returned time value
									// 51..54 (0xCC00 .. 0xD800) : PulsCount method with extended time window ( *2, *4, *8, *16)  (currently not supported)
									// bit 0..9 :
									// in PulseLen method : Debounce time in ms
									// in extended PulsCount method : switching value

	unsigned short	Attribute ;		// PulsCount method : mesuring window width in ms
									// PulsLen method : Zerotime, value must be mutliplied by factor from Controlword to get time in ms

	// Data returned from slave to Master on distributed systems 
	unsigned short	sum	;			// bits 0..13 Number of pulses counted, its always counting up / bits 14..15 : seconds counter for adjustment (future use)
	unsigned short  flow ;			// flow result : If PulseLen method, time between 2 pulses in factor * ms (factor from Controlword)
									//				 If PulseCount method, number of counts in mesuring window

	// Calibration
	// Arate and Brate are in the logical format xxx.xxx , 1/1000 for logical Flow Values without decimals. They must not exceed 999000 and only the first 3 decimal digits are considered
	// to be valid, otherwise an Overflow will occur
	unsigned long	Arate ;			// No of Pulses per logical increment, if 0, use Brate instead
	unsigned long	Brate ;			// No if logical increments per pulse, if 0, use Arate instead

	unsigned char	TimeBase ;		// Timebase for flow units : 0 = s / 1 = min / 2 = h

	unsigned long	SumDivisor;		// Adjustment for Sum, must be mutliple of 10

	// internal data for calculation
	unsigned char	method ;		// 0 = PulseCount / 1 = PulseLen
	short			lastS ;			// last counter value for sum calculation
	unsigned long	plssum ;		// this counter counts all pulses, used in some special applications
	unsigned long	zwsum ;			// main pulse counter
	unsigned char	enabled ;		// 1 : Volume counting enabled / 0 : Volume counting disabled

	// reference to local FlowMeter data
	void *			pLocal ;		// local flowmeter data, NULL if not a local flowmeter, must be casted to actual type depending on API implementation
}FLOWPAR;

/******************* FUNCTION DECLARATIONS *********************/
extern FLOWPAR FlowPars[] ;	
extern void InitFlowMeter(void);
extern void EvalFlowMeter(FLOWPAR * pPar);
extern short SetupFlowMeter(FLOWMETERSETUP *pSetup);
extern short UpdateFlowMeter(FLOWMETERUPDATE *pUpdate);
extern unsigned long cnt_GetAndResetPulssum(E_CNT *ecnt);
extern void ResetFlowCnt(E_CNT *ecnt);
extern void DisEnableFlowCnt(E_CNT *ecnt,unsigned char enabled);
extern FLOWPAR * GetFlowMeterPar(E_CNT * ecnt, E_AI * eai);

#endif

