//
// $Id: ctr_def.h 9119 2016-07-19 05:14:12Z hstrauss $
//
// Copyright (c) IMACS GmbH Germany. All rights reserved.
//
#ifndef __CTR_DEF_H__
#define __CTR_DEF_H__

#ifdef IMACS_01
  #pragma warning(disable: 4505 4710 4121 4100)
  // 4121: __unnamed : alignment of a member was sensitive to packing
  // 4100: ureferenced parameter
#endif


#ifndef ___CTR___
	#pragma warning( error: 4005 )        // Report warning as an error.
#endif

#include <limits.h>
#if CHAR_MIN != SCHAR_MIN
	#error Please set your compiler to handle char as signed char
#endif

#include "../../Def/act_typ.h"

// *************************************************************************
// Global definitions
// *************************************************************************

//************* surface interpreter ***********************
// Bitmask-Values of status information for surface interpreter
/** \brief no redraw */
#define	REDRAW_NON		 0
/** \brief partial redraw */
#define	REDRAW_PART		 1
/** \brief Redraw everything */
#define REDRAW_ALL		 2
/** \brief Triggers returning from a surface in surface-stack */
#define	EXITMODULE		 4
/** \brief Some of the following actions should be skipped */
#define	JUMPOVER		 8
/** \brief Surface was changed, not only redraw but also reevaluate conditions */
#define	NEW_SURFACE		16
/** \brief An executed key was processed */
#define	KEY_EXEC		32
/** \brief Triggers opening of a new surface on surface-stack */
#define OPENSURFACE		64

/** \brief Bitmask of REDRAW_PART || REDRAW_ALL || NEW_SURFACE */
#define REDRAW_MASK		19

//************* surface interpreter ***********************
#define MODSTEPNR 	20
#define	PARCOPY		 1
#define	PARRESTORE	 2
#define	PARRES		 3
#define	SYSCOPY		 4
#define	SYSRESTORE	 5
#define	SYSRES		 6

#define	IOTYPE_AI	 1
#define	IOTYPE_AO	 2
#define	IOTYPE_CNT	 3

#define	MESSTEXT	 1
#define	DIAGTEXT	 2

#ifdef ___CTR___
	#ifdef IMACS_01
		#define	MM_BIN		far   // Zugriffscast fuer Binaries
		#define	MM_CLASS	   // Zugriffscast fuer Objekte
		#define	NULL		0
	#endif
#endif


#define	CONDITION_DEPTH	25

//*** Macros for OSDL data handling
//
// RD_DECL_OSDL declares a local OSDL item/copy. Can be a pointer or a structure
// RD_GET_OSDL gets the data. Either by reading the data from the external memory
//   or by getting the pointer to internal memory. 
// RD_OSDL gets the deferenced data. Either simply returns the copy from the external
//   data or dereferences the pointer to the internal memory. 
#ifndef RD_GET_OSDL
  #if defined(___CTR___) && defined(RD_EXTERNAL_OSDLINI)
    #define RD_GET_OSDL(type, index, v)  RD_READ_OSDLINI((&v), BINARYALIGNMENT * (unsigned long)index, sizeof(type))
  #else // defined(___CTR___) && defined(RD_EXTERNAL_OSDLINI)
    #ifdef ___CTR___
      // RD_OSDL_DIRECT_ACCESS is used for access using $# if EMA=1 is defined in project
      #define RD_OSDL_DIRECT_ACCESS(type, index) ((type RD_MBIN *)(&osdlini[(BINARYALIGNMENT * (long)index)]))
      #define RD_GET_OSDL(type, index, var) var = RD_OSDL_DIRECT_ACCESS(type, index)
    #else // ___CTR___
      // RD_OSDL_DIRECT_ACCESS is used for access using $# if EMA=1 is defined in project
      #define RD_OSDL_DIRECT_ACCESS(type, index) ((type RD_MBIN *)(IniBinaries + (BINARYALIGNMENT * index)))
      #define RD_GET_OSDL(type, index, var) var = RD_OSDL_DIRECT_ACCESS(type, index)
    #endif // ___CTR___
  #endif // defined(___CTR___) && defined(RD_EXTERNAL_OSDLINI)
#endif // RD_GET_OSDL


#ifndef RD_DECL_OSDL
#if defined(___CTR___) && defined(RD_EXTERNAL_OSDLINI)
  // OSDL data is external
  #define RD_DECL_OSDL(type, name) type name
#else
  // OSDL data is internal.
  #define RD_DECL_OSDL(type, name) type RD_MBIN *name
#endif
#endif

#ifndef RD_OSDL_PNT
#if defined(___CTR___) && defined(RD_EXTERNAL_OSDLINI)
  // OSDL data is external
 #define RD_OSDL_PNT(type, name) type *name
#else
  // OSDL data is internal.
  #define RD_OSDL_PNT(type, name) type RD_MBIN **name
#endif
#endif

#ifndef RD_OSDL
#if defined(___CTR___) && defined(RD_EXTERNAL_OSDLINI)
  // OSDL data is external
  #define RD_OSDL(v)  (v)
#else
  // OSDL data is internal.
  #define RD_OSDL(v)  (*(v))
#endif
#endif

// *************************************************************************
// Other usefull definitions
// *************************************************************************

#define	RD_OK		0
#define RD_ERROR	-1

#define	INVALID		-2
#define	ABORT		-3
#define	ENDOF		-5
#define	NO_ENTRY	-5
#define	NORDY		-6

#define	ENOFILE		-10
#define	ENOPATH		-11 
#define	ENOMEMO		-12
#define	ENOSPAC		-13
#define	EFILE		-14
#define	EDRIVE		-15
#define	ENOMASK		-16

#define	CRLF		"\x0d\x0a"
#define	MID			0xf0

/* These defines are not used. Since they collide with settings of certain compilers, they have been removed
#define	EQU			1	// value is equal
#define	UNEQU		6	// value is bigger or smaller (unequal) 
#define	BIG			2	// value is bigger
#define	SMA			4	// value is smaller	
#define	BIGEQU		3	// value is bigger or equal	
#define	SMAEQU		5	// value is smaller or equal
*/

#define	EXT_ON		1
#define	EXT_OFF		0

// Novalues
#define	CNOVAL		((char)0x80)
#define	UCNOVAL		((unsigned char)0xFF)
#define	BNOVAL		UCNOVAL
#define	SNOVAL		((short)0x8000u)
#define	USNOVAL		((unsigned short)0xFFFFu)
#define	LNOVAL		((long)0x80000000)
#define	ULNOVAL		((unsigned long)0xFFFFFFFF)

#define	DNOVAL		((long)0)				// Date Novalue
#define	TNOVAL		((long)-1)		    	// Time Novalue


// *************************************************************************
// location definition
// *************************************************************************
#if (defined(M_I86SM) || defined(M_I86MM))
 #define LOCAT near 
#else
 #define LOCAT far
#endif

#ifdef	___CTR___	
	#define	BUFFERLEN	(MAX_CTR_TEXT_LEN + 2)
#else
	#define	BUFFERLEN	500
#endif


// *************************************************************************
// type definitions
// *************************************************************************
typedef struct 
{
	short tm_sec;	// seconds after the minute - [0,59]
	short tm_min;	// minutes after the hour - [0,59]
	short tm_hour;	// hours since midnight - [0,23]
	short tm_mday;	// day of month [1-31]
	short tm_mon;	// months since January - [0,11]
	short tm_year;	// year (if RD_RTC_EXTYEAR ist not set : yy / else yyyy)
	short tm_wday;	// days since Sunday - [0,6]
	short tm_yday;	// days since January 1 - [0,365]
	short tm_isdst;	// daylight savings time flag
} TIME;

#ifdef ___CTR___
	#define SIM_EEP_LEN	0
#else
	#define SIM_EEP_LEN	32767
#endif


// *************************************************************************
// besondere Kopierfunktionen bei Compilern die versch. Memoryfunktionen 
// besitzen (z.B. Tasking mit C167-Kernen)
// *************************************************************************

//*** falls keine besondere Funktion definiert => normale verwenden
#ifndef	RD_MEMCPY_RAMROM_CUST
	#define	RD_MemcpyRomRam(a,b,c)	memcpy((a),(b),(c))
	#define	RD_MemcpyRamRam(a,b,c)	memcpy((a),(b),(c))
#endif	




// *************************************************************************
// Multi-Tasking
// *************************************************************************
typedef void (*FUNC_PNT)(void);
typedef void (*FUNC_PNTP)(void *);

typedef struct {
	FUNC_PNT	init;
	FUNC_PNT	exec;
	short		type;			// TASK_TYPE_LOOP oder Gruppen-ID
	short		rep_time;		// [ms]
	short		priority;		// 1...   1 = hoechste
} CTR_TASK;

#define	TASK_TYPE_LOOP		1000

extern CTR_TASK SystemTaskTable[];
extern CTR_TASK ApplTaskTable[];
void startSurInterpreter(void RD_MROM *p);
void startActCtrLanguage(void);

#define SIMDLL_TASK_SLEEP	30


// *************************************************************************
// macro-definitions
// *************************************************************************
#define mk_doubform(str,l,d)    sprintf(str,"%%0%u.%ulf",l,d)
#define range(val,v1,v2)        min(max(val,v1),v2)
#define	STR(str)		        {str,(sizeof(str)-1)}



// *************************************************************************
// adress IO-definitions
// *************************************************************************
#define	TI_ON		  0x01
#define	TI_OFF		  0x00
#define	TI_DOWN		  0x02
#define	TI_UP		  0x00

#define C_LINE		     0
#define	C_HALF		     1
#define	C_FULL		     2

#define CHK_EEP		     1
#define	CHK_RAM		     2
#define	CHK_BSRAM	     3

#define	NINV		     0
#define	INV			     1
#define	IN_HARD		    -1

#define	HIS_TAB_SIZE	 8


// *************************************************************************
// 
// *************************************************************************

extern	short	abortKey;
extern	short	abortGlobalKey;
extern	short	action, vkfunktion;
extern	char	charb;
extern	long	abortTime;
extern	unsigned char	Actpwdlevel;

short	testAbort(void);
short	testGlobalKeyAbort(void);

typedef short (*CFUNC)(void);


void setActCtrLanguage(void);
extern	short	_language;

extern	CFUNC func_tab[];
extern	CFUNC Sleep_func;
extern	CFUNC ExitFunction;

void ModulInit(void);



#if ((NO_EMB_HMI == 0) && (RD_NO_EDIT_COPY == 0))
	#define PAR			ParamEd				// Editierkopie
	#define _SYSTEM		SystemEd
#else
	#define PAR		ParamData
	#define _SYSTEM	SystemData
#endif

#define	PROCESS	Process
#define _FLAG	Flags
#define LOCAL	Flags
#define EVT		Mess
#define EVAL	Flags  


//*******************************************************************
// Legacy Support
//*******************************************************************

// RD_UNITCONVERT should be used instead of UNITCONV_ON, but to support
// legacy applications, make sure that UNITCONV_ON is defined when RD_UNITCONVERT is used.
// Introduced: Jul 17, 2009. Do not remove before 2015.
#if (RD_UNITCONVERT != 0) && (! defined(UNITCONV_ON))
  #define UNITCONV_ON
#endif 

//*******************************************************************
//
//*******************************************************************
void	resetTimerCounter(void);
void	clearEAs(void);
void RdRestoreDIs(void);
void RdRestoreAIs(void);
void RdRestoreAOs(void);
void RdRestoreDOs(void);
void RdRestoreCNTs(void);
void RdRestoreTIMs(void);

void	SysInter(void);
void	SysPerm(void);
void	SysPermInit(void);

void    UserInit(void);
void	UserInter(void);
void	UserPerm(void);
void	UserPermInit(void);

#ifdef __cplusplus
extern "C" void    do_inter(void);
#else
void    do_inter(void);
#endif

short	calcChecksum(unsigned char RD_MRAM *mem, short memsize);


extern	unsigned char	RD_MRAM *TransEnabled;		// soll durch FTransEnabled ersetzt werden
extern	unsigned char	RD_MRAM *FTransEnabled;		// Transitionen der Statusmaschinen werden beachtet 0=aus, 1=ein
extern	short			RD_MRAM *FStackPos;			// akt. Stackpointer-Position
extern	long			RD_MRAM *FSysTime;	    	// pointer to system time element
extern	long			RD_MRAM *FSysDate;  		// pointer to system date element
extern	unsigned char	RD_MRAM *FAutoDLS;			// pointer to the Auto DLS var.

#ifndef ___CTR___
	extern unsigned char    simPaused; // Bit 1 SurInterpreter, Bit 2 Perm, Bit 3 Inter
	extern unsigned char    simPauseMode; // Pause-Mode: 0=no pause, 1=pause, 2=pause without sleeps
	extern void sysmem2com(void);
	extern void com2sysmem(void);
#endif // ___CTR___

// *************************************************************************
// defines fuer die standalone anwendung
// *************************************************************************
#define COMMON_EXIT     1   // Common beenden
#define COMMON_ASCIIEXP 2	// Ascii-Export der aktuellen Datenbank.
							//   arg = Nummernliste der Export-Comtypes der zu exportierenden Elemente
							//   (z.B: "134" fuer E1, E3 und E4 Elemente)  
#define COMMON_CLOSE    3	// aktuelle Aufzeichnung schliessen
#define COMMON_NEW      4	// neue Aufzeichnung erstellen+oeffnen.
							//    arg = Name der Aufzeichnung
#define COMMON_OPEN     5	// bestehende Aufzeichnung oeffnen. arg = Name der Aufzeichnung
#define COMMON_FORCE    6	// Forcemodus aktivieren/inaktivieren. arg = "0"/"1"
#define COMMON_EXECUTE  7	// andere Windowsapplikation starten: arg = Name der Aufzeichnung + Kommand

#endif // __CTR_DEF_H__
