/****************************************************************
*   file defineInitializations.h		                        *
*                                                               *
*   initialize undefined Defines to standard values, which      *
*   make sense													*
*                                                               *
* Copyright [C] by												*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/

#ifndef _DEFINEINITIALIZATIONS_H_
#define _DEFINEINITIALIZATIONS_H_

// Enable display command communication for color displays by standard
// if not disabled explicitly
#if (BITSPERPIXEL > 1)
  #ifndef RD_DISPCMD_COMMUNICATION
	#define RD_DISPCMD_COMMUNICATION    1
  #endif
#endif // (BITSPERPIXEL > 1)

// If any kind of display communication is supported set standard values
// for display communication buffer, if not already defined
#if ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0))
#ifndef DISPMEM_SIZE
#if (BITSPERPIXEL == 1)
  #if DISP_GRA != 0
    #if (VERT_NOT_HOR == 0)
      #define DISPMEM_SIZE (((DispSizeX + 7) / 8 ) * DispSizeY)
    #else
      #define DISPMEM_SIZE (((DispSizeY + 7) / 8 ) * DispSizeX)
    #endif
  #else // DISP_TXT
    // +2 Because of Cursor positions, 2nd byte per char for attribute needed by some legacy displays
    #define DISPMEM_SIZE (((DispSizeX / CHARSIZE_X) * (DispSizeY / CHARSIZE_Y) * 2) + 2)
  #endif // DISP_GRA != 0
#else // Multicolor
  #if RD_DISPCMD_COMMUNICATION == 0
	#error Color displays can only be communicated using display commands. So either set RD_DISPCMD_COMMUNICATION to 1 or disable display communication by setting RD_NO_DISPLAYCOMM to 1.
  #endif
#endif // (BITSPERPIXEL == 1)
#endif // DISPMEM_SIZE

#if ((!defined(RD_DISPCMD_BUFSIZE)) && (RD_DISPCMD_COMMUNICATION != 0))
    // This can only be a guess and should be evaluated for each project, because the needed size only depends on the design
    // It depends on the number of visualizers in a surface and also depends on the kind of visualizers in the surface. E.g.
    // for displaying a bitmap only the position and bitmap ID is needed and so bitmaps are not so space consuming in the
    // communication buffer. On the other hand extended rectangles need a lot of information and extensive use of those will
    // blow up the needed size for the display communication buffer
    #define RD_DISPCMD_BUFSIZE 1024
#endif

#endif // ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0))

// Default max retries for display command communication
#ifndef RD_DISPCMD_MAXRETRY
#define RD_DISPCMD_MAXRETRY	3
#endif

// Default buffer sizes for communication
#ifndef RD_COM_SND_BUFSIZE
#define RD_COM_SND_BUFSIZE  256
#endif // RD_COM_SND_BUFSIZE
#ifndef RD_COM_RCV_BUFSIZE
#define RD_COM_RCV_BUFSIZE  256
#endif // RD_COM_RCV_BUFSIZE

// SD card interface
#ifndef LFN_BUFFER_SIZE		// Buffer size for long filenames
#ifndef ___CTR___
#define LFN_BUFFER_SIZE	256
#else
#define LFN_BUFFER_SIZE	64
#endif
#endif

// Surface defines
#ifndef RD_SURSTACKSIZE
#define RD_SURSTACKSIZE		11		// max. number of intaves in surface-stack
#endif

#ifndef MAX_FULL_TIEFE
#define MAX_FULL_TIEFE	10
#endif

#ifndef MAX_SCREEN_TIEFE
#define MAX_SCREEN_TIEFE	10
#endif

// Define setzen, welches anzeigt, ob der Bildschirm gross genug fuer Touch-Edit-Dialoge ist
// und ob ein Touch vorhanden ist. Bildschirm > 320x240
#if TOUCH_ON != 0
	#if MAX_COLUMNS >= 40
		#if MAX_LINES >= 20
		     #if DISP_GRA != 0
				#define BIG_TOUCH
			#endif
		#endif
	#endif
#endif

//*** Systemfehlermeldung soll in der simulation immer aktiv sein
#ifndef ___CTR___
  #undef RD_ERROR_CHECK
  #define RD_ERROR_CHECK	1
#endif

#endif // _DEFINEINITIALIZATIONS_H_